################################################################################
# MTMostVisited
# A Plugin for Moveable Type
#
# Release 2.0
# January 1, 2004
#
# From Jeffrey Borlik
#
# ---------------------------------------------------------------------------
# This software is provided as-is.
# You may use it for commercial or personal use.
# If you distribute it, please keep this notice intact.
#
# Copyright (c) 2004 Jeffrey Borlik
# ---------------------------------------------------------------------------
################################################################################
## EJS ## Comments that begin like this indicate adaptations by Eric James
## EJS ## Stone in order to allow for non-numeric archive URL's.
################################################################################


package MT::Plugin::MostVisited;

use strict;
use MT::Template::Context;
use vars qw($VERSION);
$VERSION = 2.0;

# These variables do not need to be modified within this module.  It is intended
# that all of the plugin configuration will take place via the user template.

my $root = "";           # blogurl attribute.  This will be globed.
my $filetype = "html";   # filetype attribute.  This can be regex-format (like "html|php").
my $cleanbuild = 0;      # turning on cleanbuild will silently put error messages in your blog

my @searchfiles = ();    # array that will be filled with a list of logfiles
my @zipfiles = ();       # array that will be filled with a list of logfiles to be unzipped first
my $doidmap = "";				## EJS ## Variable that will contain path to ID map file
my %idmap = ();					## EJS ## Hash that will contain ID map

my %archives = ();

MT::Template::Context->add_container_tag(MostVisited => \&MostVisited);
MT::Template::Context->add_tag(MostVisitedCount => \&MostVisitedCount);
MT::Template::Context->add_tag(MostVisitedLink => \&MostVisitedLink);

################################################################################

#  Main container tag.  It loops through the most-requested entries, based
#  upon the results of the apache access log.  It can contain MTMostVisitedCount
#  and MTMostVisitedLink tags, as well as any MTEntry-type tags.'

sub MostVisited {

  require MT::Entry;
  require File::Copy;

  use Apache::ParseLog;

  my($ctx, $args, $cond) = @_;

  my $builder = $ctx->stash('builder');
  my $tokens = $ctx->stash('tokens');

  my $content = "";
  my $searchlog = ""; # local variable to hold logfile attribute

  if (exists $args->{"cleanbuild"}) { $cleanbuild = $args->{"cleanbuild"}; }
  if (exists $args->{"filetype"})   { $filetype   = $args->{"filetype"}; }
  if (exists $args->{"blogurl"})    { $root       = $args->{"blogurl"}; }

	## EJS ## This section checks whether to use an ID map

  if (exists $args->{"idmap"})	    {
		$doidmap= $args->{"idmap"};
		if (-r $doidmap) {
			open(IDMAP, $doidmap) or die "Can't open idmap due to $!.";
			while (!eof(IDMAP)){
				eval(<IDMAP>);
			}
			close(IDMAP);
		 	
		}
	}
	my $temp = %idmap;
	
	
	## EJS ## End ID map checking
	
  if (exists $args->{"logfile"})    {
     $searchlog = $args->{"logfile"};
     @searchfiles = glob($searchlog);
  }
  if (exists $args->{"ziplogfiles"}) {
     my $unzipfiles = $args->{"ziplogfiles"};
     @zipfiles = glob($unzipfiles);
  }

  if (! ((exists $args->{"blogurl"}) || (exists $args->{"idmap"}) ) ||   ## EJS ## Changed so that either blogurl or idmap can be used
      ! (exists $args->{"logfile"})  ) {

     $content = <<DONEWITHTHIS
Error in MTMostVisited.  The plugin was not configured.  A sample
format is

<ol>
<MTMostVisited blogurl=\"/blog/archives\" 
               logfile=\"/var/log/httpd/access_log*\"
               count=\"10\">
   <li><a href=\"<\$MTEntryPermalink\$>\"><\$MTEntryTitle\$></a>
       of <MTEntryDate format=\"%B %e, %Y\"> (<\$MTMostVisitedCount\$> hits)
   </li>
</MTMostVisited>
</ol>


The important configuration information is in the attribute tags of the MTMostVisited element.
\"blogurl\" is the local URL of your archive directory.  (This can include wildcards of a regular-expression
type, but paratheses are not allowed.)
\"filetype\" is the extension on your individual
archives (usually html or php).  \"logfile\" is the file location (on the server) where your log files
are located (this will be globbed via Perl, i.e. wildcards are allowed). \"count\" is the number of
entries to list.  This plugin no longer requires modification to the Perl script itself.
DONEWITHTHIS
;

    return $ctx->error($content);
  }


  my $parser = new Apache::ParseLog();
  %archives = ();

  # Load up the archives hash (filename/counts) via the
  # Apache parser.  This retrieves pages requested from
  # the $root directory, that end in $filetype (usually html).
  # Some servers (like mine) rotate their logs.

  my $numhits = 0;
  my $gotanything = 0;
  foreach my $logfile (@searchfiles) {
    my $retval = loadVisitedFromLog($logfile, $parser);
    if ($retval >= 0) {
      $numhits += $retval;
      $gotanything = 1;
    }
  }

  # Handle the zip-ed logs
  # This feature was suggested/implemented by zackvision.com

  foreach my $zipfile (@zipfiles) {
    my $tmplog = 'tmp-mtmvlog';    
    my $tmpzip = $tmplog . '.gz';

    if (! -r $zipfile) { next; }  # Silently ignore missing logfiles.  Allows globbing to work. 

    my $zerr = copy($zipfile, $tmpzip);
    if (!$zerr) {
      $content = "Error in MTMostVisited.  The plugin was unable to copy a gzipped log.  Error: $!";
      if (!$cleanbuild) { return $ctx->error($content); }
      return $content;
    }

    $zerr = `gunzip $tmpzip`;
    if ($zerr) {
      $content = "Error in MTMostVisited.  The plugin was unable to unzip the log file $zipfile.";
      if (!$cleanbuild) { return $ctx->error($content); }
      return $content;
    }

    my $retval = loadVisitedFromLog($tmplog, $parser);
    if ($retval >= 0) {
      $numhits += $retval;
      $gotanything = 1;
    }

    unlink($tmplog);
  }



  # At this point, the %archives collection will have been filled up with all of the
  # hits.  There does not need to be any additional file processing.

  # Verify that we have actually found ANY apache accesslog file.
  if (! $gotanything) {
    $content = "Error in MTMostVisited.  A webserver log was not found.
The plugin was looking for log files at \"logfile\" (i.e. \"$searchlog\").
Please double-check the location of your Apache webserver access log, and
possibly change the \"logfile\" attribute tag.";
    if (!$cleanbuild) { return $ctx->error($content); }
    return $content;
  }

  # Verify that we found ANY hit via the $root regex
  if (scalar keys %archives == 0) {
    $content = "Error in MTMostVisited.  NO hits on MT archives were found in the
webserver logs (some webserver logs were found).  The plugin determines if it was a MT archive hit by matching the
page hits against the \"blogurl\" attribute.  In your case, it looks for hits that look
like <tt> http://yoursite.whatever$root/000001.$filetype </tt>.  If that URL
doesn't look right for your individual archives, then you will either need to
change your \"blogurl\" value or the \"filetype\" variable in the MTMostVisited tag.  Note that
the \"blogurl\" variable should not have the http/domain thing in front of it, and should
not end in a \"/\".  I.e. it should look like <tt> /archives </tt> or something like that.
(It is also possible that you really haven't had any hits on your individual archives.
In that case, this plugin is probably not very useful to you.)";
    if (!$cleanbuild) { return $ctx->error($content); }
    return $content;
  }
  
  # do stuff with the archives hash (filename/counts)

  my $inum = 0;
  my $imax = 10;  # default
  if (exists $args->{"count"}) { $imax = $args->{"count"}; }

  foreach my $entryid (sort { $archives{$b} <=> $archives{$a} } keys %archives) {

    # stash something
    $ctx->stash('mostvisited_count', $archives{$entryid});
    $ctx->stash('mostvisited_entryid', $entryid);

    # load entry stuff

    # Check to ensure that this entry does not have non-digit characters
    # in it, as that would make it a month archive or a popup-image page.
    # Simply skip that one.
    if ($entryid =~ /\D/) {
      next;
    }

    #ASSUMPTION:  The name of the page is the same as the entry_id.

    my $entry = MT::Entry->load($entryid);

    #TODO: Need filtering to ensure that it is actually an entry for the current
    #      blog, but that doesn't seem too critical as different blogs probably
    #      have different filepaths
    if (! $entry) {
      $content = "Error in MTMostVisited, while loading EntryID \"$entryid\": ";
      $content = $content . "This does not appear to be a valid EntryID.  It should be all numbers.";
      if (! $cleanbuild) { return $ctx->error( $content ); }
      return $content;
#			next;
    }

    local $ctx->{__stash}{entry} = $entry;
    local $ctx->{current_timestamp} = $entry->created_on;

    # build internal stuff
    my $out = $builder->build($ctx, $tokens);
    if (! $out ) {
      $content = "Error in MTMostVisited, while building EntryID \"$entryid\": ";
      $content = $content . $builder->errstr;
      $content = "  Perhaps you are using a tag unsupported by the MTMostVisited collection.";
      if (! $cleanbuild) { return $ctx->error($content); }
      return $content;
    }

    $content .= $out;

    last if (++$inum >= $imax);
  }

  if ($content eq "") {
    my $num = scalar keys %archives;
    $content = "No hits on entry archive files.  There were $num pages found
that match the pattern http://your.url.tld$root/000001.$filetype.";
    if ($num > 0) {
      $content = $content . "  They
must have been skipped due to extraneous characters in the filename.  (Are your
individual archives named with the EntryID?  If not, this plugin will not work.  Bug
the author about it.)\n"; }
    else {
      $content = $content . "  Perhaps the plugin is misconfigured, and the \"blogurl\"
variable isn't set correctly.  It should be the web address/URL of your individual
archives, without the http://your.url.tld on the front, e.g. \"/archives\".  Or
maybe your website actually hasn't gotten any hits on archive files yet.";
    }

  }

  return $content;
}

#############################################

sub MostVisitedCount {
  my $ctx = shift;
  return $ctx->stash('mostvisited_count');
}

#############################################

sub MostVisitedLink {
  my $ctx = shift;
  return $root . "/" . $ctx->stash('mostvisited_entryid') . "." . $filetype;
}



#############################################
# loadVisitedFromLog - Utility subroutine, that parses an apache access
# log, and appends to the %archives hash.  It uses the Apache::ParseLog perlmodule.

sub loadVisitedFromLog {
  my $log = shift;
  my $parser = shift;

  if (! -r $log) { return -1; }

  $parser = $parser->config(transferlog => "$log");

  my $transferlog = $parser->getTransferLog();

  my %files = $transferlog->file();

  my $numfound = 0;
  foreach (keys %files) {


    # This regex is used to determine if a given hit (from the log) is
    # a hit to an entry.  The $root variable and the $filetype variable
    # can both be regular expressions.  (Thanks to Eric James Stone
    # for the suggestion, in the MT Forums.)  By design, this subroutine
    # does not care if the entries are numeric or not.  (Maybe someday
    # I'll figure out a way of allowing non-EntryID individual archive
    # filenames.)
		
		## EJS ## Added so that if $doidmap has a value, use the ID Map for checking the log file.
	if ($doidmap)	    {																									## EJS ##
		while (my ($key, $value) = each (%idmap)) {												## EJS ## Get the ID mapping.
			if (m@^$key$@) {																								## EJS ## If $key matches the file, then then entry ID is $value
				++$numfound;																									## EJS ##
				my $count = 0;																								## EJS ##
				if (exists $archives{$value}) { $count = $archives{$value}; } ## EJS ##
				$archives{$value} = $count + $files{$_};											## EJS ##
			}																																## EJS ##
		}																																	## EJS ##
	} elsif (/^($root)\/(\w+)\.($filetype)$/) {													## EJS ## Changed from an if to an elseif in order to continue from the above

      ++$numfound;
      my $count = 0;
      if (exists $archives{$2}) { $count = $archives{$2}; }
      $archives{$2} = $count + $files{$_};

    }
  }

  return $numfound;
}

1;
